@@ -3,7 +3,8 @@
 #'
 #' This is essentially a wrapper around the \code{spades} call that allows for multiple
 #' calls to spades. This function will use a single processor, or multiple processors if
-#' \code{\link[raster]{beginCluster}} has been run first.
+#' \code{\link[raster]{beginCluster}} has been run first or a cluster objects is passed
+#' in the \code{cl} argument (gives more control to user).
 #'
 #' Generally, there are 2 reasons to do this: replication and varying simulation inputs
 #' to accomplish some sort of simulation experiment. This function deals with both of these
@@ -53,17 +54,18 @@
 #'            all. See details.
 #'
 #' @inheritParams spades
+#' @inheritParams splitRaster
 #' @details
-#' This function requires a complete simList: this simList will form the basis of the
-#' modifications as passed by params, modules, inputs, and objects.
-#' All params, modules, inputs or objects
-#' passed into this function will override the corresponding params, modules, inputs,
-#' or identically named objects that are in the \code{sim} argument.
+#' This function requires a complete simList: this simList will form the basis of
+#' the modifications as passed by params, modules, inputs, and objects.
+#' All params, modules, inputs or objects passed into this function will override
+#' the corresponding params, modules, inputs, or identically named objects that
+#' are in the \code{sim} argument.
 #'
 #' This function is parallel aware, using the same mechanism as used in the \code{raster}
 #' package. Specifically, if you start a cluster using \code{\link{beginCluster}}, then
 #' this experiment function will automatically use that cluster. It is always a good
-#' idea to stop the cluster when finished, using \code{\link{endCluster}}. See
+#' idea to stop the cluster when finished, using \code{\link{endCluster}}.
 #'
 #' Here are generic examples of how \code{params}, \code{modules}, \code{objects},
 #' and \code{inputs} should be structured.
@@ -81,14 +83,14 @@
 #'                    stringsAsFactors = FALSE)
 #'   )}
 #'
-#'
 #' Output directories are changed using this function: this is one of the dominant
 #' side effects of this function. If there are only replications, then a set of
 #' subdirectories will be created, one for each replicate.
-#' If there are varying parameters and or modules, \code{outputPath} is updated to include
-#' a subdirectory for each level of the experiment. These are not nested, i.e., even if there
-#' are nested factors, all subdirectories due to the experimental setup will be
-#' at the same level. Replicates will be one level below this.
+#' If there are varying parameters and or modules, \code{outputPath} is updated
+#' to include a subdirectory for each level of the experiment.
+#' These are not nested, i.e., even if there are nested factors, all subdirectories
+#' due to the experimental setup will be at the same level.
+#' Replicates will be one level below this.
 #' The subdirectory names will include the module(s), parameter names, the parameter values,
 #' and input index number (i.e., which row of the inputs data.frame).
 #' The default rule for naming is a concatenation of:
@@ -129,27 +131,30 @@
 #' \code{outputPath} before the call to experiment.
 #'
 #' \code{dirPrefix} can be used to give custom names to directories for outputs.
-#' There is a special value, \code{"simNum"}, that is used as default, which is an arbitrary
-#' number  associated with the experiment.
+#' There is a special value, \code{"simNum"}, that is used as default, which is
+#' an arbitrary number  associated with the experiment.
 #' This corresponds to the row number in the \code{attr(sims, "experiment")}.
 #' This \code{"simNum"} can be used with other strings, such as
-#' \code{dirPrefix = c("expt","simNum")}.
+#' \code{dirPrefix = c("expt", "simNum")}.
 #'
-#' The experiment structure is kept in two places: the return object has an attribute, and a file
-#' named \code{experiment.RData} (see argument \code{experimentFile}) located in \code{outputPath(sim)}.
+#' The experiment structure is kept in two places: the return object has an attribute,
+#' and a file named \code{experiment.RData} (see argument \code{experimentFile})
+#' located in \code{outputPath(sim)}.
 #'
-#' \code{substrLength}, if \code{0}, will eliminate the subfolder naming convention and use
-#' only \code{dirPrefix}.
+#' \code{substrLength}, if \code{0}, will eliminate the subfolder naming convention
+#' and use only \code{dirPrefix}.
 #'
-#' If \code{cache = TRUE} is passed, then this will pass this to \code{spades}, with the
-#' additional argument \code{replicate = x}, where x is the replicate number. That means that
-#' if a user runs \code{experiment} with \code{replicate = 4} and \code{cache = TRUE}, then spades
-#' will run 4 replicates, caching the results, including replicate = 1, replicate = 2, replicate = 3,
-#' and replicate = 4. Thus, if a second call to experiment with the exact same simList is passed,
-#' and \code{replicates = 6}, the first 4 will be taken from the cached copies, and replicate 5 and
-#' 6 will be run (and cached) as normal. If \code{notOlderThan} used with a time that is more recent
-#' than the cached copy, then a new spades will be done, and the cached copy will be deleted from
-#' the cache repository, so there will only ever be one copy of a particular replicate for a
+#' If \code{cache = TRUE} is passed, then this will pass this to \code{spades},
+#' with the additional argument \code{replicate = x}, where x is the replicate number.
+#' That means that if a user runs \code{experiment} with \code{replicate = 4} and
+#' \code{cache = TRUE}, then SpaDES will run 4 replicates, caching the results,
+#' including replicate = 1, replicate = 2, replicate = 3, and replicate = 4.
+#' Thus, if a second call to experiment with the exact same simList is passed,
+#' and \code{replicates = 6}, the first 4 will be taken from the cached copies,
+#' and replicate 5 and 6 will be run (and cached) as normal.
+#' If \code{notOlderThan} used with a time that is more recent than the cached copy,
+#' then a new spades will be done, and the cached copy will be deleted from the
+#' cache repository, so there will only ever be one copy of a particular replicate for a
 #' particular simList. NOTE: caching may not work as desired on a Windows machine because
 #' the sqlite database can only be written to one at a time, so there may be collisions.
 #'
@@ -173,233 +178,14 @@
 #'
 #' @author Eliot McIntire
 #'
-#' @examples
-#' \dontrun{
-#'  library(magrittr) # use %>% in a few examples
-#'
-#'  tmpdir <- file.path(tempdir(), "examples")
-#'
-#' # Create a default simList object for use through these examples
-#'   mySim <- simInit(
-#'     times = list(start = 0.0, end = 2.0, timeunit = "year"),
-#'     params = list(
-#'       .globals = list(stackName = "landscape", burnStats = "nPixelsBurned"),
-#'       # Turn off interactive plotting
-#'       fireSpread = list(.plotInitialTime = NA),
-#'       caribouMovement = list(.plotInitialTime = NA),
-#'       randomLandscapes = list(.plotInitialTime = NA)
-#'     ),
-#'     modules = list("randomLandscapes", "fireSpread", "caribouMovement"),
-#'     paths = list(modulePath = system.file("sampleModules", package = "SpaDES"),
-#'                  outputPath = tmpdir),
-#'     # Save final state of landscape and caribou
-#'     outputs = data.frame(objectName = c("landscape", "caribou"), stringsAsFactors = FALSE)
-#'   )
-#'
-#' # Example 1 - test alternative parameter values
-#'   # Create an experiment - here, 2 x 2 x 2 (2 levels of 2 params in fireSpread,
-#'   #    and 2 levels of 1 param in caribouMovement)
-#'
-#'   # Here is a list of alternative values for each parameter. They are length one
-#'   #   numerics here -- e.g., list(0.2, 0.23) for spreadprob in fireSpread module,
-#'   #   but they can be anything, as long as it is a list.
-#'   experimentParams <- list(fireSpread = list(spreadprob = list(0.2, 0.23),
-#'                                              nFires = list(20, 10)),
-#'                             caribouMovement = list(N = list(100, 1000)))
-#'
-#'   sims <- experiment(mySim, params = experimentParams)
-#'
-#'   # see experiment:
-#'   attr(sims, "experiment")
-#'
-#'   # Read in outputs from sims object
-#'   FireMaps <- do.call(stack, lapply(1:NROW(attr(sims, "experiment")$expDesign),
-#'                      function(x) sims[[x]]$landscape$Fires))
-#'   if (interactive()) Plot(FireMaps, new = TRUE)
-#'
-#'   # Or reload objects from files, useful if sim objects too large to store in RAM
-#'   caribouMaps <- lapply(sims, function(sim) {
-#'     caribou <- readRDS(outputs(sim)$file[outputs(sim)$objectName == "caribou"])
-#'   })
-#'   names(caribouMaps) <- paste0("caribou",1:8)
-#'   # Plot does not plot whole lists (yet)
-#'   for (i in 1:NROW(attr(sims,"experiment")$expDesign)) {
-#'     if (interactive()) Plot(caribouMaps[[i]], size = 0.1)}
-#'
-#' # Example 2 - test alternative modules
-#'   # Example of changing modules, i.e., caribou with and without fires
-#'   # Create an experiment - here, 2 x 2 x 2 (2 levels of 2 params in fireSpread,
-#'   #    and 2 levels of 1 param in caribouMovement)
-#'   experimentModules <- list(
-#'       c("randomLandscapes", "fireSpread", "caribouMovement"),
-#'       c("randomLandscapes", "caribouMovement"))
-#'   sims <- experiment(mySim, modules = experimentModules)
-#'   attr(sims, "experiment")$expVals # shows 2 alternative experiment levels
-#'
-#' # Example 3 - test alternative parameter values and modules
-#'   # Note, this isn't fully factorial because all parameters are not
-#'   #   defined inside smaller module list
-#'   sims <- experiment(mySim, modules = experimentModules, params = experimentParams)
-#'   attr(sims, "experiment")$expVals # shows 10 alternative experiment levels
-#'
-#' # Example 4 - manipulate manipulate directory names -
-#'   #  "simNum" is special value for dirPrefix, it is converted to 1, 2, ...
-#'   sims <- experiment(mySim, params = experimentParams, dirPrefix = c("expt", "simNum"))
-#'   attr(sims, "experiment")$expVals # shows 10 alternative experiment levels, 26 unique
-#'                                    #   parameter values
-#'
-#' # Example 5 - doing replicate runs -
-#'   sims <- experiment(mySim, replicates = 2)
-#'   attr(sims, "experiment")$expDesign # shows 2 replicates of same experiment
-#'
-#' # Example 6 - doing replicate runs, but within a sub-directory
-#'   sims <- experiment(mySim, replicates = 2, dirPrefix = c("expt"))
-#'   lapply(sims, outputPath) # shows 2 replicates of same experiment, within a sub directory
-#'
-#' # Example 7 - doing replicate runs, of a complex, non factorial experiment.
-#'   # Here we do replication, parameter variation, and module variation all together.
-#'   # This creates 20 combinations.
-#'   # The experiment function tries to make fully factorial, but won't
-#'   # if all the levels don't make sense. Here, changing parameter values
-#'   # in the fireSpread module won't affect the simulation when the fireSpread
-#'   # module is not loaded:
-#'   # library(raster)
-#'   # beginCluster(20) # if you have multiple clusters available, use them here to save time
-#'   sims <- experiment(mySim, replicates = 2, params = experimentParams,
-#'                     modules = experimentModules,
-#'                     dirPrefix = c("expt", "simNum"))
-#'   # endCluster() # end the clusters
-#'   attr(sims, "experiment")
-#'
-#' # Example 8 - Use replication to build a probability map.
-#'   # For this to be meaningful, we need to provide a fixed input landscape,
-#'   #   not a randomLandscape for each experiment level. So requires 2 steps.
-#'   # Step 1 - run randomLandscapes module twice to get 2 randomly
-#'   #  generated landscape maps. We will use 1 right away, and we will
-#'   #  use the two further below
-#'   mySimRL <- simInit(
-#'      times = list(start = 0.0, end = 0.1, timeunit = "year"),
-#'      params = list(
-#'        .globals = list(stackName = "landscape"),
-#'        # Turn off interactive plotting
-#'        randomLandscapes = list(.plotInitialTime = NA)
-#'      ),
-#'      modules = list("randomLandscapes"),
-#'      paths = list(modulePath = system.file("sampleModules", package = "SpaDES"),
-#'                   outputPath = file.path(tmpdir, "landscapeMaps1")),
-#'      outputs = data.frame(objectName = "landscape", saveTime = 0, stringsAsFactors = FALSE)
-#'   )
-#'   # Run it twice to get two copies of the randomly generated landscape
-#'   mySimRLOut <- experiment(mySimRL, replicate = 2)
-#'
-#'   #extract one of the random landscapes, which will be passed into next as an object
-#'   landscape <- mySimRLOut[[1]]$landscape
-#'
-#'   # here we don't run the randomLandscapes module; instead we pass in a landscape
-#'   #  as an object, i.e., a fixed input
-#'   mySimNoRL <- simInit(
-#'      times = list(start = 0.0, end = 1, timeunit = "year"), # only 1 year to save time
-#'      params = list(
-#'        .globals = list(stackName = "landscape", burnStats = "nPixelsBurned"),
-#'        # Turn off interactive plotting
-#'        fireSpread = list(.plotInitialTime = NA),
-#'        caribouMovement = list(.plotInitialTime = NA)
-#'      ),
-#'      modules = list("fireSpread", "caribouMovement"), # No randomLandscapes modules
-#'      paths = list(modulePath = system.file("sampleModules", package = "SpaDES"),
-#'                   outputPath = tmpdir),
-#'      objects = c("landscape"), # Pass in the object here
-#'      # Save final state (the default if saveTime is not specified) of landscape and caribou
-#'      outputs = data.frame(objectName = c("landscape", "caribou"), stringsAsFactors = FALSE)
-#'   )
-#'
-#'   # Put outputs into a specific folder to keep them easy to find
-#'   outputPath(mySimNoRL) <- file.path(tmpdir, "example8")
-#'   sims <- experiment(mySimNoRL, replicates = 8) # Run experiment
-#'   attr(sims, "experiment") # shows the experiment, which in this case is just replicates
-#'
-#'   # list all files that were saved called 'landscape'
-#'   landscapeFiles <- dir(outputPath(mySimNoRL), recursive = TRUE, pattern = "landscape",
-#'                         full.names = TRUE)
-#'
-#'   # Can read in Fires layers from disk since they were saved, or from the sims
-#'   #  object
-#'   # Fires <- lapply(sims, function(x) x$landscape$Fires) %>% stack
-#'   Fires <- lapply(landscapeFiles, function(x) readRDS(x)$Fires) %>% stack
-#'   Fires[Fires > 0] <- 1 # convert to 1s and 0s
-#'   fireProb <- sum(Fires)/nlayers(Fires) # sum them and convert to probability
-#'   if (interactive()) Plot(fireProb, new = TRUE)
-#'
-#' # Example 9 - Pass in inputs, i.e., input data objects taken from disk
-#'   #  Here, we, again, don't provide randomLandscapes module, so we need to
-#'   #  provide an input stack called lanscape. We point to the 2 that we have
-#'   #  saved to disk in Example 8
-#'   mySimInputs <- simInit(
-#'      times = list(start = 0.0, end = 2.0, timeunit = "year"),
-#'      params = list(
-#'        .globals = list(stackName = "landscape", burnStats = "nPixelsBurned"),
-#'        # Turn off interactive plotting
-#'        fireSpread = list(.plotInitialTime = NA),
-#'        caribouMovement = list(.plotInitialTime = NA)
-#'      ),
-#'      modules = list("fireSpread", "caribouMovement"),
-#'      paths = list(modulePath = system.file("sampleModules", package = "SpaDES"),
-#'                   outputPath = tmpdir),
-#'      # Save final state of landscape and caribou
-#'      outputs = data.frame(objectName = c("landscape", "caribou"), stringsAsFactors = FALSE)
-#'   )
-#'   landscapeFiles <- dir(tmpdir, pattern = "landscape_year0", recursive = TRUE, full.names = TRUE)
-#'
-#'   # Varying inputs files - This could be combined with params, modules, replicates also
-#'   outputPath(mySimInputs) <- file.path(tmpdir, "example9")
-#'   sims <- experiment(mySimInputs,
-#'      inputs = lapply(landscapeFiles,function(filenames) {
-#'        data.frame(file = filenames, loadTime = 0, objectName = "landscape",
-#'                   stringsAsFactors = FALSE) })
-#'    )
-#'
-#'   # load in experimental design object
-#'   experiment <- load(file = file.path(tmpdir, "example9", "experiment.RData")) %>% get()
-#'   print(experiment) # shows input files and details
-#'
-#' # Example 10 - Use a very simple output dir name using substrLength = 0,
-#'   #   i.e., just the simNum is used for outputPath of each spades call
-#'   outputPath(mySim) <- file.path(tmpdir, "example10")
-#'   sims <- experiment(mySim, modules = experimentModules, replicates = 2,
-#'                      substrLength = 0)
-#'   lapply(sims, outputPath) # shows that the path is just the simNum
-#'   experiment <- load(file = file.path(tmpdir, "example10", "experiment.RData")) %>% get()
-#'   print(experiment) # shows input files and details
-#'
-#' # Example 11 - use clearSimEnv = TRUE to remove objects from simList
-#'   # This will shrink size of return object, which may be useful because the
-#'   #  return from experiment function may be a large object (it is a list of
-#'   # simLists). To see size of a simList, you have to look at the objects
-#'   #  contained in the  envir(simList).  These can be obtained via objs(sim)
-#'   sapply(sims, function(x) object.size(objs(x))) %>% sum + object.size(sims)
-#'   # around 3 MB
-#'   # rerun with clearSimEnv = TRUE
-#'   sims <- experiment(mySim, modules = experimentModules, replicates = 2, substrLength = 0,
-#'                      clearSimEnv = TRUE)
-#'   sapply(sims, function(x) object.size(objs(x))) %>% sum + object.size(sims)
-#'   # around 250 kB, i.e., all the simList contents except the objects.
-#'
-#' # Example 12 - pass in objects
-#'   experimentObj <- list(landscape = lapply(landscapeFiles, readRDS) %>%
-#'                                     setNames(paste0("landscape",1:2)))
-#'   # Pass in this list of landscape objects
-#'   sims <- experiment(mySimNoRL, objects = experimentObj)
-#'
-#' # Remove all temp files
-#'   unlink(tmpdir, recursive = TRUE)
-#' }
+#' @example inst/examples/example_experiment.R
 #'
 setGeneric(
   "experiment",
   function(sim, replicates = 1, params, modules, objects = list(), inputs,
            dirPrefix = "simNum", substrLength = 3, saveExperiment = TRUE,
            experimentFile = "experiment.RData", clearSimEnv = FALSE, notOlderThan,
-           ...) {
+           cl, ...) {
     standardGeneric("experiment")
 })
 
@@ -409,17 +195,22 @@ setMethod(
   signature(sim = "simList"),
   definition = function(sim, replicates, params, modules, objects, inputs,
                         dirPrefix, substrLength, saveExperiment,
-                        experimentFile, clearSimEnv, notOlderThan, ...) {
+                        experimentFile, clearSimEnv, notOlderThan, cl, ...) {
 
     if (missing(params)) params <- list()
-    if (missing(modules)) modules <- list(unlist(SpaDES::modules(sim)[-(1:4)]))
+    if (missing(modules)) modules <- list(unlist(SpaDES::modules(sim)))
     if (missing(inputs)) inputs <- list()
     if (missing(objects)) {objects <- list() } else if (length(objects) == 1) {
       objects <- unlist(objects, recursive = FALSE)
     }
 
-    cl <- tryCatch(getCluster(), error = function(x) NULL)
-    on.exit(if (!is.null(cl)) returnCluster())
+    if (missing(cl)) {
+      cl <- tryCatch(getCluster(), error = function(x) NULL)
+      on.exit(if (!is.null(cl)) returnCluster(), add = TRUE)
+    }
+
+    # cl <- tryCatch(getCluster(), error = function(x) NULL)
+    # on.exit(if (!is.null(cl)) returnCluster(), add = TRUE)
 
     #if (length(modules) == 0) modules <- list(modules(sim)[-(1:4)])
     factorialExpList <- lapply(seq_along(modules), function(x) {
@@ -462,11 +253,11 @@ setMethod(
     # Add replicates to experiment
     if (replicates > 1) {
       if (length(replicates == 1)) {
-        replicates = seq_len(replicates)
+        replicates <- seq_len(replicates)
       }
       factorialExp <- do.call(rbind, replicate(length(replicates), factorialExp,
                                                simplify = FALSE))
-      factorialExp$replicate = rep(replicates, each = numExpLevels)
+      factorialExp$replicate <- rep(replicates, each = numExpLevels)
     }
 
     FunDef <- function(ind, ...) {
@@ -548,14 +339,13 @@ setMethod(
 
         if ("modules" %in% names(factorialExp)) {
           if (!identical(sort(unlist(modules[factorialExp[ind, "modules"]])),
-                         sort(unlist(SpaDES::modules(sim)[-(1:4)])))) { # test if modules are different from sim,
+                         sort(unlist(SpaDES::modules(sim))))) { # test if modules are different from sim,
             #  if yes, rerun simInit
             sim_ <- simInit(params = params(sim_),
                             modules = as.list(unlist(modules[factorialExp[ind, "modules"]])),
                             times = append(lapply(times(sim_)[2:3], as.numeric), times(sim_)[4]),
                             paths = paths(sim_),
                             outputs = outputs(sim_))
-
           }
         }
       }
@@ -635,6 +425,13 @@ setMethod(
       if (!is.na(pmatch("Windows", Sys.getenv("OS")))) {
         parallel::clusterEvalQ(cl, library(SpaDES))
       }
+      packagesToLoad <- SpaDES::packages(sim)
+      parallel::clusterExport(cl, "packagesToLoad", envir = environment())
+      parallel::clusterEvalQ(cl, {
+        lapply(packagesToLoad, library, character.only = TRUE)
+      })
+      
+      
     } else {
       parFun <- "lapply"
       args <- list(X = 1:NROW(factorialExp), FUN = FunDef)
