@@ -15,8 +15,12 @@ defineModule(sim, list(
     defineParameter(".plotInitialTime", "numeric", NA_real_, NA, NA, "This describes the simulation time at which the first plot event should occur"),
     defineParameter(".saveInitialTime", "numeric", NA_real_, NA, NA, "This describes the simulation time at which the first save event should occur")
   ),
-  inputObjects = data.frame(objectName = NA_character_, objectClass = NA_character_, sourceURL = NA_character_, other = NA_character_, stringsAsFactors = FALSE),
-  outputObjects = data.frame(objectName = NA_character_, objectClass = NA_character_, other = NA_character_, stringsAsFactors = FALSE)
+  inputObjects = bind_rows(
+    expectsInput(objectName = NA_character_, objectClass = NA_character_,
+                 desc = NA_character_, sourceURL = NA_character_, other = NA_character_)),
+  outputObjects = bind_rows(
+    createsOutput(objectName = NA_character_, objectClass = NA_character_, desc = NA_character_, other = NA_character_)
+  )
 ))
 
 ### no other code is needed for this module group
