@@ -22,24 +22,25 @@ defineModule(sim, list(
   documentation = list(),
   reqdPkgs = list("grid", "raster", "sp"),
   parameters = rbind(
-    defineParameter("moveInitialTime", "numeric", 1.0, NA, NA, "time to schedule first movement event"),
+    defineParameter("moveInitialTime", "numeric", start(sim)+1, NA, NA, "time to schedule first movement event"),
     defineParameter("moveInterval", "numeric", 1.0, NA, NA, "time interval between movoment events"),
-    defineParameter("N", "numeric", 100L, NA, NA, "initial number of caribou"),
+    defineParameter("N", "numeric", 100L, 10L, 1000L, "initial number of caribou"),
     defineParameter("torus", "logical", FALSE, NA, NA, "should the map wrap around like a torus?"),
-    defineParameter(".plotInitialTime", "numeric", 0, NA, NA, "time to schedule first plot event"),
+    defineParameter(".plotInitialTime", "numeric", start(sim), NA, NA, "time to schedule first plot event"),
     defineParameter(".plotInterval", "numeric", 1, NA, NA, "time interval between plot events"),
     defineParameter(".saveInitialTime", "numeric", NA_real_, NA, NA, "time to schedule first save event"),
     defineParameter(".saveInterval", "numeric", NA_real_, NA, NA, "time interval between save events")
   ),
-  inputObjects = data.frame(
-    objectName = globals(sim)$stackName, objectClass = "RasterStack",
-    sourceURL = NA_character_, other = "layername = \"habitatQuality\"",
-    stringsAsFactors = FALSE),
-  outputObjects = data.frame(
-    objectName = c(globals(sim)$stackName, "caribou"),
-    objectClass = c("RasterStack", "SpatialPointsDataFrame"),
-    other = c("layername = \"habitatQuality\"", NA_character_),
-    stringsAsFactors = FALSE)
+  inputObjects = bind_rows(
+    expectsInput(objectName = globals(sim)$stackName, objectClass = "RasterStack",
+                 desc = "layername = \"habitatQuality\"", sourceURL = NA_character_)
+  ),
+  outputObjects = bind_rows(
+    createsOutput(objectName = globals(sim)$stackName, objectClass = "RasterStack",
+                  desc = "layername = \"habitatQuality\""),
+    createsOutput(objectName = "caribou", objectClass = "SpatialPointsDataFrame",
+                  desc = NA_character_)
+  )
 ))
 
 ## event types
@@ -53,11 +54,13 @@ doEvent.caribouMovement <- function(sim, eventTime, eventType, debug = FALSE) {
     sim <- sim$caribouMovementInit(sim)
 
     # schedule the next event
-    sim <- scheduleEvent(sim, params(sim)$caribouMovement$moveInitialTime,
+    sim <- scheduleEvent(sim, P(sim)$moveInitialTime,
                          "caribouMovement", "move")
-    sim <- scheduleEvent(sim, params(sim)$caribouMovement$.plotInitialTime,
+    #sim <- scheduleEvent(sim, P(sim)$moveInitialTime,
+    #                     "caribouMovement", "move")
+    sim <- scheduleEvent(sim, P(sim)$.plotInitialTime,
                          "caribouMovement", "plot.init", .last())
-    sim <- scheduleEvent(sim, params(sim)$caribouMovement$.saveInitialTime,
+    sim <- scheduleEvent(sim, P(sim)$.saveInitialTime,
                          "caribouMovement", "save", .last()+1)
   } else if (eventType == "move") {
     # do stuff for this event
@@ -65,7 +68,7 @@ doEvent.caribouMovement <- function(sim, eventTime, eventType, debug = FALSE) {
 
     # schedule the next event
     sim <- scheduleEvent(sim, time(sim) +
-                           params(sim)$caribouMovement$moveInterval,
+                           P(sim)$moveInterval,
                          "caribouMovement", "move")
   } else if (eventType == "plot.init") {
     # do stuff for this event
@@ -75,7 +78,7 @@ doEvent.caribouMovement <- function(sim, eventTime, eventType, debug = FALSE) {
 
     # schedule the next event
     sim <- scheduleEvent(sim, time(sim) +
-                           params(sim)$caribouMovement$.plotInterval,
+                           P(sim)$.plotInterval,
                          "caribouMovement", "plot", .last())
   } else if (eventType == "plot") {
     # do stuff for this event
@@ -86,7 +89,7 @@ doEvent.caribouMovement <- function(sim, eventTime, eventType, debug = FALSE) {
 
     # schedule the next event
     sim <- scheduleEvent(sim, time(sim) +
-                           params(sim)$caribouMovement$.plotInterval,
+                           P(sim)$.plotInterval,
                          "caribouMovement", "plot", .last())
   } else if (eventType == "save") {
     # do stuff for this event
@@ -94,7 +97,7 @@ doEvent.caribouMovement <- function(sim, eventTime, eventType, debug = FALSE) {
 
     # schedule the next event
     sim <- scheduleEvent(sim, time(sim) +
-                           params(sim)$caribouMovement$.saveInterval,
+                           P(sim)$.saveInterval,
                          "caribouMovement", "save", .last()+1)
 
   } else {
@@ -114,7 +117,7 @@ caribouMovementInit <- function(sim) {
               xmax(sim[[globals(sim)$stackName]]))
 
   # initialize caribou agents
-  N <- params(sim)$caribouMovement$N
+  N <- P(sim)$N
   IDs <- as.character(1:N)
   sex <- sample(c("female", "male"), N, replace=TRUE)
   age <- round(rnorm(N, mean=8, sd=3))
@@ -148,7 +151,7 @@ caribouMovementMove <- function(sim) {
   sim$caribou <- move("crw", agent = sim$caribou,
                       extent = extent(sim[[globals(sim)$stackName]]),
                       stepLength = ln, stddev = sd, lonlat = FALSE,
-                      torus = params(sim)$caribouMovement$torus)
+                      torus = P(sim)$torus)
 
   return(invisible(sim))
 }
